Imports Microsoft.VisualBasic
Imports System.Collections
Imports System.Web

Namespace DLAFormfactory
    ''' <summary>
    ''' class with container of dlaparameters
    ''' is used to transport a collection of value/name combinations through the layers
    ''' </summary>
    ''' <remarks></remarks>
    Public Class DlaParameterContainer
        Private objCol As New Collection()
        ''' <summary>
        ''' Add a (DLA) parameter to the collection
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <param name="strValue"></param>
        ''' <remarks></remarks>
        Public Sub AddParameter(ByVal strName As String, ByVal strValue As String)
            Me.AddParameter(strName, strValue, "string")
        End Sub
        ''' <summary>
        ''' Add a (DLA) parameter to the collection
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <param name="strValue"></param>
        ''' <param name="strType"></param>
        Public Sub AddParameter(ByVal strName As String, ByVal strValue As String, ByVal strType As String)
            'strValue = strValue.Replace("'", "''")
            If Not Me.FindKey(strName) Then
                Me.objCol.Add(New DLAParameter(strName, strValue, strType), strName.ToLower())
            End If
        End Sub
        ''' <summary>
        ''' Update the parameter based on the name
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <param name="strValue"></param>
        ''' <param name="strType"></param>
        Public Sub UpdateParameter(ByVal strName As String, ByVal strValue As String, ByVal strType As String)
            'strValue = strValue.Replace("'", "''")
            If Me.objCol.Contains(strName) Then
                Me.objCol.Remove(strName)
            End If
            Me.objCol.Add(New DLAParameter(strName, strValue, strType), strName.ToLower())
        End Sub
        ''' <summary>
        ''' Get a parameter value based on its name
        ''' and clear it if it's a NULL value
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetItemValue4Control(ByVal strName As String) As String
            Dim strRet As String

            strRet = Me.GetItemValue(strName)
            If strRet.ToUpper() = "NULL" Then
                strRet = ""
            End If
            Return strRet
        End Function
        ''' <summary>
        ''' See if a key exists or not
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function FindKey(ByVal strName As String) As Boolean
            Dim blnFound As Boolean
            Dim objPara As DLAParameter
            For Each objPara In Me.objCol
                If objPara.GetName.ToUpper = strName.ToUpper Then
                    blnFound = True
                    Exit For
                End If
            Next
            Return blnFound
        End Function
        ''' <summary>
        ''' return the number of items in the collection
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetItemCount() As Integer
            Return Me.objCol.Count
        End Function
        ''' <summary>
        ''' Get a parameter value based on its name
        ''' </summary>
        ''' <param name="index"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetItemName(ByVal index As Integer) As String
            Dim objPara As DLAParameter

            objPara = CType(Me.objCol.Item(index), DLAParameter)
            Return objPara.GetName()
        End Function
        ''' <summary>
        ''' Get a parameter value based on its name
        ''' </summary>
        ''' <param name="strName"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function GetItemValue(ByVal strName As String) As String
            Dim objPara As DLAParameter
            Dim strValue As String
            Try
                objPara = CType(Me.objCol.Item(strName.ToLower()), DLAParameter)
                strValue = CType(objPara.GetValue(), String)
                strValue = strValue.Replace("''", "'")
                Return strValue
            Catch e As Exception
                DLA2EAHelper.Error2Log(e)
                Return e.Message
            End Try
        End Function
        ''' <summary>
        ''' Process a sql statement to a statement with values
        ''' based on a simple replace routine
        ''' </summary>
        ''' <param name="strSql"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Function ProcessStatement(ByVal strSql As String) As String
            Dim objPara As DLAParameter

            strSql = strSql.Replace("@@", "#")
            For Each objPara In Me.objCol
                strSql = strSql.Replace(objPara.GetIdentifier(), objPara.GetSqlValue())
            Next
            strSql = strSql.Replace("is_less", "<=")
            strSql = strSql.Replace("is_more", ">=")
            strSql = strSql.Replace("isless", "<=")
            strSql = strSql.Replace("ismore", ">=")
            '          strSql = strSql.Replace("#weblevelid#", DLASessionCookie.GetLoginLevel())
            '           strSql = strSql.Replace("#activelanguage#", DLASessionCookie.GetLanguage())
            strSql = strSql.Replace("'NULL'", "NULL")
            Return strSql
        End Function
    End Class
End Namespace
