﻿Imports System.IO
Imports TEA.DLAFormfactory

''' <summary>
''' Class for processing definitions, such as connections, queries, etc.
''' Is done based on a Dataset that is stored via XML in the config file
''' </summary>
Public Class IDEADefinitions
    Protected oIDEADefinitions As DataSet
    Sub New()
        Me.LoadDefinitions()
        Me.LoadFromSettings(GetObject(, "EA.App").Repository)
    End Sub

    Sub New(Repository As EA.Repository)
        Me.LoadDefinitions()
        Me.LoadFromSettings(Repository)
    End Sub
    ''' <summary>
    ''' Check for version number when the version nummber in the settings is higher you can not use the AddOn
    ''' </summary>
    ''' <returns></returns>
    Function CheckVersion() As Boolean
        Return Me.GetSettingValue("IDEAVERSION") <= My.Application.Info.Version.ToString().Substring(0, 3)
    End Function
    ''' <summary>
    ''' Loading the metamodel of the definitions in the form of a dataset with datatables
    ''' </summary>
    Sub LoadDefinitions()
        Try
            Me.oIDEADefinitions = New DataSet("Definitions")
            Dim objSettings As DataTable
            objSettings = New DataTable("Settings")
            objSettings.Columns.Add(New DataColumn("Name", System.Type.GetType("System.String")))
            objSettings.Columns.Add(New DataColumn("Type", System.Type.GetType("System.String")))
            objSettings.Columns.Add(New DataColumn("Value", System.Type.GetType("System.String")))
            Me.oIDEADefinitions.Tables.Add(objSettings)

            Dim objSQL As DataTable
            objSQL = New DataTable("SQL-Statement")
            objSQL.Columns.Add(New DataColumn("Name", System.Type.GetType("System.String")))
            objSQL.Columns.Add(New DataColumn("Type", System.Type.GetType("System.String")))
            objSQL.Columns.Add(New DataColumn("Statement", System.Type.GetType("System.String")))
            objSQL.Columns.Add(New DataColumn("Template", System.Type.GetType("System.String")))
            Me.oIDEADefinitions.Tables.Add(objSQL)
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
    End Sub
    ''' <summary>
    ''' Save the definitions to the users setting or to an artifact (IDEAConfig) in the current repository
    ''' </summary>
    ''' <param name="Settings"></param>
    ''' <param name="statements"></param>
    ''' <param name="Repository"></param>
    ''' <returns></returns>
    Public Function SaveSettings(Settings As String, statements As String, Repository As EA.Repository) As Boolean
        Dim oDT As DataTable
        Dim oElement As EA.Element
        oDT = DLA2EAHelper.SQL2DataTable(String.Format("SELECT object_id FROM t_object WHERE object_type = 'Artifact' AND name = 'IDEAConfig' AND version >= '{0}'", My.Application.Info.Version.ToString().Substring(0, 3)), Repository)
        If oDT.Rows.Count > 0 Then
            Dim oDR As DataRow
            oDR = oDT.Rows(0)
            oElement = Repository.GetElementByID(oDR("object_id"))
            Dim objDS As New DataSet2Repository()
            objDS.AddOrUpdateTaggedValue(oElement, "Settings", Settings, True)
            objDS.AddOrUpdateTaggedValue(oElement, "SQL-Statement", statements, True)
            oElement.Update()
        Else
            My.Settings.SettingsTable = Settings
            My.Settings.StatementsTable = statements
            My.Settings.Save()
        End If
        Return True
    End Function
    ''' <summary>
    ''' Load from settings this can be an IDEASettings artifact in the repository or via the user settings
    ''' </summary>
    Sub LoadFromSettings(Repository As EA.Repository)
        Dim oDT As DataTable
        Try
            oDT = DLA2EAHelper.SQL2DataTable(String.Format("SELECT t_objectproperties.notes as config, t_objectproperties.property as name FROM t_object, t_objectproperties WHERE t_object.object_type = 'Artifact'  AND t_object.name = 'IDEAConfig' AND t_object.[version] >= '{0}' AND t_object.object_id = t_objectproperties.object_id", My.Application.Info.Version.ToString().Substring(0, 3)), Repository)
            If oDT.Rows.Count = 0 Then
                LoadDataSetXML(My.Settings.SettingsTable, "", "Settings")
                LoadDataSetXML(My.Settings.StatementsTable, "", "SQL-Statement")
            Else
                For Each oRow As DataRow In oDT.Rows
                    Dim strText As String
                    strText = oRow("config")
                    LoadDataSetXML(strText, "", oRow("Name"))
                Next
                If Not CheckVersion() Then
                    MsgBox("You are using an older version please upgrade by installing a current version")
                End If
            End If
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
    End Sub
    ''' 
    ''' 
    ''' <summary>
    ''' Loading the dataset with definitions from the configuration file f from the fill of the result tab.
    ''' If no material is found, a few sample items will be made
    ''' </summary>
    ''' <param name="xmlstring">Dataset definities uit de config file</param>
    ''' <param name="importstring">Dataset afkomstig vanuit het result scherm</param>
    Sub LoadDataSetXML(xmlstring As String, importstring As String, Tablename As String)
        Try
            Me.oIDEADefinitions.Tables(Tablename).Rows.Clear()
            If xmlstring.Length > 0 Or importstring.Length > 0 Then
                Dim oSR As New StringReader(IIf(importstring.Length > 10, importstring, xmlstring))
                Me.oIDEADefinitions.ReadXml(oSR, XmlReadMode.IgnoreSchema)
            End If
            'volgens mij is deze niet nodig en verder wel essentieel
            '            If Tablename = "Settings" And Me.oIDEADefinitions.Tables(Tablename).Rows.Count <= 1 Then
            System.IO.Directory.CreateDirectory("c:\idea")

            Me.AddInitialSetting("PDFPackageTemplate", "Diagram Report", "HTML")
            Me.AddInitialSetting("PDFDiagramTemplate", "Diagram Report", "HTML")
            Me.AddInitialSetting("PDFElementTemplate", "Diagram Report", "HTML")
            Me.AddInitialSetting("LogFile", "c:\idea\error.log", "FILES")
            Me.AddInitialSetting("ReleaseDirectory", "c:\idea", "FILES")
            Me.AddInitialSetting("HTMLPath", "c:\idea\", "HTML")
            'path for the WPP diagram name for direct export when saving ther diagram
            Me.AddInitialSetting("WPP_Diagram_File", "?", "HTML")
            Me.AddInitialSetting("HTMLTemplate", "c:\idea\templates\basis_template.html", "HTML")
            Me.AddInitialSetting("TemplateReplaceValue", "DEFAULT", "AID")
            Me.AddInitialSetting("ExcelConnection", "Provider=Microsoft.ACE.OLEDB.12.0;Data Source=[FILE];Extended Properties=""Excel 12.0 Xml;HDR=YES"";", "EXCEL")
            Me.AddInitialSetting("ElementPackageSQL", "SELECT * FROM t_object where package_id = #package_id# ORDER BY name", "RTF")
            Me.AddInitialSetting("DiagramPackageSQL", "SELECT * FROM t_diagram where package_id = #package_id# ORDER BY name", "RTF")
            Me.AddInitialSetting("ElementDiagramSQL", "SELECT * FROM t_diagramobjects where diagram_id = #diagram_id# ORDER BY RectTop, RectLeft", "RTF")
            Me.AddInitialSetting("ConnectionString", "Provider=SQLOLEDB;Integrated Security=SSPI;Persist Security Info=False;Initial Catalog=DLAdministratie2018;Data Source=DESKTOP-HNMVP79;Use Procedure for Prepare=1;Auto Translate=True;Packet Size=4096;Workstation ID=DESKTOP-HNMVP79;Use Encryption for Data=False;Tag with column collation when possible=False;", "FORMFACTORY")
            Me.AddInitialSetting("DeduplicateElementsPackage", "Select orig.name, orig.stereotype, orig.author, orig.version, dupl.name as duplicate_name, t_package.name as package_name, dupl.author as duplicate_author, orig.object_id as origid, dupl.object_id as duplid From t_object orig, t_package, t_object dupl where dupl.package_id = t_package.package_id and orig.name = dupl.name and orig.object_id <> dupl.object_id and (orig.stereotype = dupl.stereotype or (orig.stereotype IS NULL AND dupl.stereotype IS NULL)) and orig.object_type = dupl.object_type and orig.object_type NOT IN('Package', 'Diagram', 'Note', 'Text', 'ProxyConnector') and orig.name IS NOT NULL and orig.name <> '' and orig.version = dupl.version and orig.package_id IN(#package_id#) order by 1 ", "DEDUPL")
            Me.AddInitialSetting("DeduplicateElementElement", "Select dupl.name as duplicate_name, t_package.name as package_name, dupl.author as duplicate_author, orig.object_id as origid, dupl.object_id as duplid From t_object orig, t_package, t_object dupl where dupl.package_id = t_package.package_id and orig.name = dupl.name and orig.object_id <> dupl.object_id and (orig.stereotype = dupl.stereotype or (orig.stereotype IS NULL AND dupl.stereotype IS NULL)) and orig.object_type = dupl.object_type and orig.object_type NOT IN('Package', 'Diagram', 'Note', 'Text', 'ProxyConnector') and orig.name IS NOT NULL and orig.name <> '' and orig.version = dupl.version and orig.object_id = #object_id# order by 1 ", "DEDUPL")
            Me.AddInitialSetting("DeduplicateConnectorsPackage", "Select distinct startobj.name As object_name, orig.name, orig.connector_type,  dupl.name As dupl_name, destobj.name as dupl_object_name, destobj.author as dupl_author, dupl.direction, orig.connector_id as origid, dupl.connector_id as duplid From t_connector orig, t_connector dupl, t_object startobj, t_object destobj Where ((orig.name = dupl.name) Or (orig.name Is null And dupl.name Is null)) And orig.connector_type = dupl.connector_type And orig.end_object_id = dupl.end_object_id And orig.start_object_id = dupl.start_object_id And orig.start_object_id = startobj.Object_ID And orig.end_object_ID = destobj.object_id And Orig.direction = dupl.direction And (orig.styleex IS NULL OR orig.styleex not like '%LFSP=%') And (orig.stereotype = dupl.stereotype Or orig.stereotype Is NULL) And orig.Connector_ID <> dupl.connector_id And startobj.package_id IN(#package_id#) ", "DEDUPL")
            Me.AddInitialSetting("DeduplicateConnectorsDiagram", "Select orig.name, orig.stereotype, orig.author, orig.version, dupl.name as duplicate_name, t_package.name as package_name, dupl.author as duplicate_author, orig.object_id as origid, dupl.object_id as duplid From t_object orig, t_package, t_object dupl, t_diagramobjects
                where dupl.package_id = t_package.package_id 
                and orig.name = dupl.name 
                and orig.object_id <> dupl.object_id 
                and (orig.stereotype = dupl.stereotype or (orig.stereotype IS NULL AND dupl.stereotype IS NULL)) 
                and orig.object_type = dupl.object_type 
                and orig.object_type NOT IN('Package', 'Diagram', 'Note', 'Text', 'ProxyConnector', 'Grouping', 'Boundary') 
                and orig.name IS NOT NULL and orig.name <> '' 
                and orig.version = dupl.version 
                and orig.object_id = t_diagramobjects.object_id
                and t_diagramobjects.Diagram_ID = #diagram_id# 
                order by 1  ", "DEDUPL")

            Me.AddInitialSetting("DeduplicateElementsDiagram", "Select orig.name, orig.stereotype, orig.author, orig.version, dupl.name as duplicate_name, 
                t_package.name as package_name, dupl.author as duplicate_author, orig.object_id as origid, dupl.object_id as duplid 
                From t_object orig, t_package, t_object dupl, t_diagramobjects 
                where dupl.package_id = t_package.package_id 
                and orig.name = dupl.name and orig.object_id <> dupl.object_id 
                 and (orig.stereotype = dupl.stereotype or (orig.stereotype IS NULL AND dupl.stereotype IS NULL)) 
                and orig.object_type = dupl.object_type and orig.object_type NOT IN('Package', 'Diagram', 'Note', 'Text', 'ProxyConnector') 
                and orig.name IS NOT NULL and orig.name <> '' 
                and orig.version = dupl.version 
                and orig.object_id = t_diagramobjects.object_id
                and t_diagramobjects.diagram_id IN(#diagram_id#) order by 1 ", "DEDUPL")

            Me.AddInitialSetting("WasteBinPackage_id", "-999", "WASTEBIN")
            Me.AddInitialSetting("IDEAVersion", My.Application.Info.Version.ToString().Substring(0, 3), "VERSION")
            Me.AddInitialSetting("TaggedValuePrefix", "IDEA", "CONFIG")

            'Wizard elementen
            Me.AddInitialSetting("WizardTemplateSQL", "select child.package_id, child.name
                                    from t_package parent 
                                    inner join t_package child on child.parent_id = parent.package_id
                                    where parent.ea_guid = '{C44D8B99-E93D-480b-9DC1-D3846F9CEBDF}'
                                    order by child.name", "WIZARD")

            Me.AddInitialSetting("WizardWorkPackageSQL", "select child.package_id, child.name
                                    from t_package parent 
                                    inner join t_package child on child.parent_id = parent.package_id
                                    where parent.name = 'Persoonlijke mappen'
                                    order by child.name", "WIZARD")

            Me.AddInitialSetting("WizardDiagramObjectsSQL", "UPDATE t_diagramobjects 
                                    SET t_diagramobjects.object_id = WorkObject.Object_id 
                                    FROM t_diagramobjects INNER JOIN  t_object TemplateObject ON  t_diagramobjects.Object_ID = TemplateObject.Object_ID
                                    INNER JOIN t_object WorkObject ON TemplateObject.Name = WorkObject.Name
                                    INNER JOIN t_diagram ON   t_diagramobjects.diagram_id = t_diagram.diagram_id
                                    WHERE t_diagram.package_id = {1}
                                    AND TemplateObject.Package_ID = {0}
                                    AND WorkObject.Package_ID = {1} ", "WIZARD")

            Me.AddInitialSetting("WizardUnusedPatternsSQL", "DELETE 
                                    FROM t_diagramobjects 
                                    WHERE object_id IN({0}) AND diagram_id = {1}", "WIZARD")

            Me.AddInitialSetting("WizardPreceedingModulesSQL", "SELECT DISTINCT PreceedingModule.Name as PreceedingModuleName
                                    , PreceedingParent.Name as PreceedingParentName
                                    , ParentEntity.Name as ParentName
                                    , ActiveEntity.Name as ActiveEntityName
                                    , PreceedingModule.Status
                                    , PreceedingModule.Object_id
                                    FROM t_object as ActiveEntity
                                    , t_object as ParentEntity
                                    , t_object as PreceedingParent
                                    , t_object as PreceedingModule
                                    , t_connector as Active_Parent
                                    , t_connector as Parent_PreceedingParent
                                    , t_connector as PreceedingParent_PreceedingModule
                                    WHERE ActiveEntity.object_id IN({0})
                                    AND ActiveEntity.object_id = Active_Parent.Start_Object_ID
                                    AND ParentEntity.object_id = Active_Parent.End_Object_ID
                                    AND Active_Parent.Stereotype = 'ArchiMate_Specialization'
                                    AND ParentEntity.Name LIKE '%#DIA%'
                                    AND ParentEntity.object_id = Parent_PreceedingParent.End_Object_ID
                                    AND PreceedingParent.object_id = Parent_PreceedingParent.Start_Object_ID
                                    AND PreceedingParent.Stereotype = 'ArchiMate_ApplicationService'
                                    AND (PreceedingParent.Name LIKE '%#DIA%' OR PreceedingParent.Name LIKE '%#NS%')
                                    AND Parent_PreceedingParent.Stereotype = 'ArchiMate_Triggering'
                                    AND PreceedingParent_PreceedingModule.Stereotype = 'ArchiMate_Specialization'
                                    AND PreceedingParent.object_id =PreceedingParent_PreceedingModule.End_Object_ID
                                    AND PreceedingModule.object_id = PreceedingParent_PreceedingModule.Start_Object_ID
                                    AND PreceedingModule.Stereotype = 'ArchiMate_ApplicationService'
                                    AND PreceedingModule.Name LIKE '%#Module%' 
                                    AND PreceedingModule.status IN('Referentie architectuur', 'Solution architectuur', '#werkmap#')", "WIZARD")

            Me.AddInitialSetting("WizardPatternsSQL", " SELECT DISTINCT Pattern.Name as PatternName
                                , ParentEntity.Name as ParentName
                                , ActiveEntity.Name as ActiveEntityName
                                , Pattern.Object_id
                                FROM t_object as ActiveEntity
                                , t_object as ParentEntity
                                , t_object as Pattern
                                , t_connector as Active_Parent
                                , t_connector as Parent_Pattern
                                WHERE ActiveEntity.object_id in({0})
		                        AND ActiveEntity.object_id = Active_Parent.Start_Object_ID
                                AND ParentEntity.object_id = Active_Parent.End_Object_ID
                                AND Active_Parent.Stereotype = 'ArchiMate_Specialization'
                                AND (ParentEntity.Name LIKE '%#DIA%' OR ParentEntity.Name LIKE '%#NS%')
                                AND ParentEntity.object_id = Parent_Pattern.Start_Object_ID
                                AND Pattern.object_id = Parent_Pattern.End_Object_ID
                                AND Pattern.Stereotype = 'ArchiMate_ApplicationService'
                                AND Pattern.Name LIKE '%#Patroon%' 
                                AND Parent_Pattern.Stereotype = 'ArchiMate_Aggregation' 
                                AND Pattern.Name NOT LIKE '%#DIA%'", "WIZARD")

            'vullen van de sql statements configs
            If Tablename = "SQL-Statement" And Me.oIDEADefinitions.Tables(Tablename).Rows.Count <= 1 Then
                'DMF raamwerk
                Me.AddInitialSQL("MappingColumn", "DMF",
                                       "SELECT DISTINCT CASE WHEN t_connector.name IS NULL THEN t_connector.name ELSE concat('_',t_connector.name) END AS connectorname, t_connector.direction, attrsource.name AS sourcecolumn , attrtarget.name +  CASE WHEN targetprops.value = 'H'  THEN '_BK' +  CASE WHEN t_connector.name IS NULL  THEN ''  ELSE concat('_',t_connector.name)  END  ELSE '' END AS targetcolumn , classsource.name AS sourcetable, classtarget.name AS targettable, t_diagram.name AS diagram_name, targetprops.value AS tableType FROM t_connector JOIN t_diagramlinks ON t_diagramlinks.ConnectorID = t_connector.connector_id JOIN t_diagram ON t_diagram.Diagram_ID = t_diagramlinks.DiagramID JOIN t_object AS classsource ON classsource.object_id = t_connector.start_object_id JOIN t_attribute AS attrsource ON attrsource.object_id = classsource.object_id JOIN t_object AS classtarget ON classtarget.object_id = t_connector.end_object_id JOIN t_attribute AS attrtarget ON attrtarget.object_id = classtarget.object_id LEFT OUTER JOIN t_objectproperties AS targetprops ON targetprops.Object_ID = classtarget.Object_ID WHERE t_diagram.diagram_id = #diagram_id# AND t_connector.styleex LIKE '%LFSP=' + attrsource.ea_guid + '%' AND t_connector.styleex LIKE '%LFEP=' + attrtarget.ea_guid + '%' AND targetprops.Property = 'TableType' ",
                                       "INSERT INTO MGT.DynamicLoadDataVault_AttributeMappingInput (LoadDate,UserName,SourceTable,SourceAttribute,Rol,StageName,TargetTable,TargetAttribute) VALUES (CONVERT(DATETIME2(6),GETDATE(),121),CURRENT_USER,'#sourcetable#','#sourcecolumn#','#connectorname#','#sourcetable#_vw','#targettable#','#targetcolumn#') ")
                Me.AddInitialSQL("MappingTable", "DMF", "SELECT DISTINCT classsource.name  AS sourcetable, dbo.nsda_gettaggedvalue(classsource.Object_ID, 'Owner') AS sourceschema , LEFT(classsource.name, CHARINDEX('_', classsource.name)-1) AS sourcerecordsource , classtarget.name  AS targettable , dbo.nsda_gettaggedvalue(classtarget.Object_ID, 'Owner') AS targetschema , CASE WHEN t_connector.name IS NULL THEN t_connector.name ELSE CONCAT('_',t_connector.name) END  AS connectorname , (SELECT t_connectorconstraint.notes  FROM t_connectorconstraint WHERE connectorid = t_connector.connector_id )  AS connectorfilter FROM t_connector, t_diagramlinks, t_diagram, t_object AS classsource, t_object AS classtarget WHERE t_connector.connector_id = t_diagramlinks.ConnectorID AND t_diagramlinks.DiagramID = t_diagram.Diagram_ID AND t_connector.start_object_id = classsource.object_id AND t_connector.end_object_id = classtarget.object_id AND t_connector.styleex IS NULL AND t_diagram.diagram_id = #diagram_id# ",
                                       "INSERT INTO MGT.DynamicLoadDataVault_MappingInput (LoadDate,UserName,RecordSource,SourceSchema,SourceTableName,StageSchema,StageName,TargetSchema,TargetTableName,Rol,SourceFilter,CreateView) VALUES (CONVERT(DATETIME2(6),GETDATE(),121),CURRENT_USER,'#sourcerecordsource#','#sourceschema#','#sourcetable#','#sourceschema#','#sourcetable#_vw','#targetschema#','#targettable#','#connectorname#','#connectorfilter#','1')")

                'Housekeeping
                Me.AddInitialSQL("HousekeepingBewaarTermijn", "Housekeeping",
                                 "select bom_object.bom_id
                            , bom_object.bom_naam
                            , bom_object.bewaartermijn
                            , bom_object.bewaartermijneenheid
                            , bom_object.bewaartermijnvanafwanneer
                            , table_object.object_name as [tabelnaam] 
                            , dbo.nsda_gettaggedvalue(table_object.object_id, 'Owner') as [tableschema]
                            from nsda_object as [ldm_object] 
                            , nsda_bombewaartermijn as [bom_object] 
                            , nsda_object as [table_object] 
                            , nsda_connector as [flcon]
                            , nsda_connector as [lccon]
                            , t_diagramobjects as tablediagramobjects 
                            where flcon.end_object_id = ldm_object.object_id 
                            and flcon.start_object_id = table_object.object_id
                            and lccon.start_object_id = ldm_object.object_id 
                            and lccon.end_object_id = bom_object.bom_id 
                            and ldm_object.object_type = 'Class' 
                            and table_object.object_type = 'table' 
                            and ldm_object.object_name is not null 
                            and table_object.object_id = tablediagramobjects.object_id 
                            and tablediagramobjects.diagram_id = #diagram_id#",
                                "INSERT INTO [PBD].[META_FBDOM_DV_Relaties] ([FBDOM_id], [FBDOMNaam], [bewaartermijn], [bewaartermijneenheid], [bewaartermijnvanafwanneer], [Tabelowner],  [Tabelnaam] ) 
                            VALUES ('#bom_id#', '#bom_naam#', '#bewaartermijn#', '#bewaartermijneenheid#', '#bewaartermijnvanafwanneer#', '#tableschema#',  '#tabelnaam#' );")

                Me.AddInitialSQL("HousekeepingBOM", "Housekeeping",
                                 "select bom_object.bom_id
                            , bom_object.bom_naam
                            , table_object.object_name as [tabelnaam] 
                            , dbo.nsda_gettaggedvalue(table_object.object_id, 'Owner') as [tableschema]
                            from nsda_object as [ldm_object] 
                            , nsda_bombewaartermijn as [bom_object] 
                            , nsda_object as [table_object] 
                            , nsda_connector as [flcon]
                            , nsda_connector as [lccon]
                            , t_diagramobjects as tablediagramobjects 
                            where flcon.end_object_id = ldm_object.object_id 
                            and flcon.start_object_id = table_object.object_id
                            and lccon.start_object_id = ldm_object.object_id 
                            and lccon.end_object_id = bom_object.bom_id 
                            and ldm_object.object_type = 'Class' 
                            and table_object.object_type = 'table' 
                            and ldm_object.object_name is not null 
                            and table_object.object_id = tablediagramobjects.object_id 
                            and tablediagramobjects.diagram_id = #diagram_id#",
                                "INSERT INTO [PBD].[META_FBDOM_DV_Relaties] ([FBDOM_id], [FBDOMNaam], [Tabelowner],  [Tabelnaam] ) 
                            VALUES ('#bom_id#', '#bom_naam#', '#tableschema#',  '#tabelnaam#' );")

                Me.AddInitialSQL("HousekeepingPDM", "Housekeeping",
                                     "select distinct dbo.nsda_gettaggedvalue(parent_object.object_id, 'Owner') as [parentowner]
                            , parent_object.object_name as [parentname]
                            , dbo.nsda_gettaggedvalue(child_object.object_id, 'Owner') as [childowner]
                            , child_object.object_name as [childname] 
                            from [nsda_object] as [parent_object], nsda_object as [child_object]
                            , nsda_connector as [con], t_diagramobjects as parentdiagramlinks
                            , t_diagramobjects as childdiagramlinks 
                            where con.start_object_id = child_object.object_id 
                            and con.end_object_id = parent_object.object_id 
                            and parent_object.object_type = 'table' 
                            and child_object.object_type = 'table' 
                            and con.destcard = '1' 
                            and parent_object.object_name is not null 
                            and parent_object.object_id = parentdiagramlinks.object_id 
                            and parentdiagramlinks.diagram_id = #diagram_id# 
                            and child_object.object_id = childdiagramlinks.object_id 
                            and childdiagramlinks.diagram_id = #diagram_id# 
                            union 
                            select distinct dbo.nsda_gettaggedvalue(child_object.object_id, 'Owner') 
                            , child_object.object_name as [childname]
                            , dbo.nsda_gettaggedvalue(parent_object.object_id, 'Owner') 
                            , parent_object.object_name as [parentname]
                            from [nsda_object] as [parent_object], nsda_object as [child_object], nsda_connector as [con]
                            , t_diagramobjects as parentdiagramlinks, t_diagramobjects as childdiagramlinks
                            where con.end_object_id = child_object.object_id 
                            and con.start_object_id = parent_object.object_id 
                            and parent_object.object_type = 'table' 
                            and child_object.object_type = 'table' 
                            and con.destcard = '1' 
                            and parent_object.object_name is not null 
                            and parent_object.object_id = parentdiagramlinks.object_id 
                            and parentdiagramlinks.diagram_id = #diagram_id# 
                            and child_object.object_id = childdiagramlinks.object_id 
                            and childdiagramlinks.diagram_id = #diagram_id#",
                                    "INSERT INTO [PBD].[META_DV_TABELRELATIES] ([parentowner], [parentname], [childowner],  [childname] ) 
                            VALUES ('#parentowner#', '#parentname#',  '#childowner#',  '#childname#' );")
            End If
        Catch ex As Exception
            MsgBox("Error in setting config", MsgBoxStyle.OkOnly)
        End Try

    End Sub

    ''' <summary>
    ''' Search in a dataset collection for a filter, activate it and return the filtered subset
    ''' </summary>
    ''' <param name="table">Name of the datatable to retrieve data from</param>
    ''' <param name="filter">Filterstring in the datatable</param>
    ''' <returns></returns>
    Function GetFilteredTable(table As String, filter As String) As DataTable
        Dim oDV As DataView
        Try
            oDV = New DataView(Me.oIDEADefinitions.Tables(table))
            oDV.RowFilter = filter
            Return oDV.ToTable()
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
        Return Nothing
    End Function

    ''' <summary>
    ''' utility routine to get the value of a configitem based on its name
    ''' If the name is not found, an empty string is returned
    ''' </summary>
    ''' <param name="name">Name of configuration item</param>
    ''' <returns>Value of the configuration item</returns>
    Function GetSettingValue(name As String) As String
        Dim DT As DataTable
        DT = Me.GetFilteredTable("SETTINGS", String.Format("name = '{0}' ", name))
        If DT.Rows.Count > 0 Then
            Dim Row As DataRow
            Row = DT.Rows(0)
            Return Row("Value")
        End If
        Return ""
    End Function
    ''' <summary>
    ''' Retrieve datatable based on the configuration datatable name
    ''' </summary>
    ''' <param name="table"></param>
    ''' <returns></returns>
    Function GetTable(table As String) As DataTable
        Try
            Return Me.oIDEADefinitions.Tables(table)
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
        Return Nothing
    End Function
    ''' <summary>
    ''' Create the initial settings for the IDEA AddOn so later the user can modify it via the settings window
    ''' </summary>
    ''' <param name="Name"></param>
    ''' <param name="Value"></param>
    ''' <param name="Type"></param>
    Sub AddInitialSetting(Name As String, Value As String, Optional Type As String = "Config")
        If Me.GetSettingValue(Name).Length = 0 Then
            Me.AddSetting(Name, Value)
        End If
    End Sub
    ''' <summary>
    ''' Create an initial SQL statement for the diagram helper screen
    ''' </summary>
    ''' <param name="Name"></param>
    ''' <param name="Type"></param>
    ''' <param name="Statement"></param>
    ''' <param name="Template"></param>
    Sub AddInitialSQL(Name As String, Type As String, Statement As String, Template As String)
        If Me.GetFilteredTable("SQL-Statement", String.Format(" Name = '{0}' ", Name)).Rows.Count = 0 Then
            Me.AddStatement(Name, Type, Statement, Template)
        End If
    End Sub
    ''' <summary>
    ''' Add a setting to the table settings
    ''' </summary>
    ''' <param name="Name"></param>
    ''' <param name="Value"></param>
    ''' <param name="Type"></param>
    Sub AddSetting(Name As String, Value As String, Optional Type As String = "Config")
        Try
            Dim oRow As DataRow
            oRow = Me.oIDEADefinitions.Tables("Settings").NewRow()
            oRow("Name") = Name
            oRow("Value") = Value
            oRow("Type") = Type
            Me.oIDEADefinitions.Tables("Settings").Rows.Add(oRow)
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
    End Sub
    ''' <summary>
    ''' Add an item to the datatable with statements
    ''' </summary>
    ''' <param name="Name"></param>
    ''' <param name="Type"></param>
    ''' <param name="Statement"></param>
    ''' <param name="Template"></param>
    Sub AddStatement(Name As String, Type As String, Statement As String, Template As String)
        Try
            Dim oRow As DataRow
            oRow = Me.oIDEADefinitions.Tables("SQL-Statement").NewRow()
            oRow("Name") = Name
            oRow("Type") = Type
            oRow("Statement") = Statement
            oRow("Template") = Template
            Me.oIDEADefinitions.Tables("SQL-Statement").Rows.Add(oRow)
        Catch ex As Exception
            DLA2EAHelper.Error2Log(ex)
        End Try
    End Sub
End Class
